@@ -10,32 +10,24 @@ import time
 from shakenfist import artifact
 from shakenfist.baseobject import (DatabaseBackedObject as dbo,
                                    active_states_filter)
+from shakenfist.baseobjectmapping import OBJECT_NAMES_TO_CLASSES
 from shakenfist import blob
 from shakenfist.config import config
-from shakenfist.constants import OBJECT_NAMES
 from shakenfist.daemons import daemon
 from shakenfist import etcd
 from shakenfist import exceptions
 from shakenfist import instance
 from shakenfist import logutil
-from shakenfist import net
-from shakenfist import networkinterface
+from shakenfist import network
 from shakenfist.node import (
     Node, Nodes,
     active_states_filter as node_active_states_filter,
     inactive_states_filter as node_inactive_states_filter,
     nodes_by_free_disk_descending)
-from shakenfist.tasks import FetchBlobTask
+from shakenfist.upload import Uploads
 
 
 LOG, _ = logutil.setup(__name__)
-OBJECT_NAMES_TO_CLASSES = {
-    'artifact': artifact.Artifact,
-    'blob': blob.Blob,
-    'instance': instance.Instance,
-    'network': net.Network,
-    'networkinterface': networkinterface.NetworkInterface
-}
 
 
 class Monitor(daemon.Daemon):
@@ -60,21 +52,51 @@ class Monitor(daemon.Daemon):
     def _cluster_wide_cleanup(self, last_loop_run):
         LOG.info('Running cluster maintenance')
 
+        # Recompute our cache of what blobs are on what nodes every 30 minutes
+        if time.time() - last_loop_run > 1800:
+            per_node = defaultdict(list)
+            with etcd.ThreadLocalReadOnlyCache():
+                for b in blob.Blobs([active_states_filter]):
+                    for node in b.locations:
+                        per_node[node].append(b.uuid)
+
+            for node in Nodes([]):
+                node.blobs = per_node.get(node.uuid, [])
+
         # Cleanup soft deleted objects
-        for objtype in OBJECT_NAMES:
+        for objtype in OBJECT_NAMES_TO_CLASSES:
             for _, objdata in etcd.get_all(objtype, None):
                 try:
                     obj = OBJECT_NAMES_TO_CLASSES[objtype].from_db(
                         objdata['uuid'])
                     if (obj.state.value == dbo.STATE_DELETED and
                             time.time() - obj.state.update_time > config.CLEANER_DELAY):
-                        LOG.with_object(obj).info('Hard deleting')
                         obj.hard_delete()
                 except exceptions.BadObjectVersion:
                     LOG.with_fields({
-                        objtype: obj.uuid
+                        objtype: objdata['uuid']
                     }).warning('Could not load object for hard delete, bad version')
 
+        # Cleanup vxids which specify a missing network
+        for k, objdata in etcd.get_all('vxlan', None):
+            network_uuid = objdata.get('network_uuid')
+            if network_uuid:
+                n = network.Network.from_db(network_uuid)
+                if not n:
+                    etcd.WrappedEtcdClient().delete(k)
+                    LOG.with_fields({
+                        'network': network_uuid,
+                        'vxid record': k
+                    }).warning('Cleaning up leaked vxlan')
+
+        # Cleanup old uploads which were never completed
+        for upload in Uploads([]):
+            if time.time() - upload.state.update_time > 7 * 24 * 3600:
+                LOG.with_fields({
+                    'upload': upload.uuid
+                }).warning('Cleaning up stale upload')
+                upload.hard_delete()
+
         # Prune artifacts which might have too many versions
         for a in artifact.Artifacts([]):
             a.delete_old_versions()
@@ -96,36 +118,22 @@ class Monitor(daemon.Daemon):
             absent_nodes.append(n.fqdn)
         LOG.info('Found %d inactive nodes' % len(absent_nodes))
 
-        current_fetches = defaultdict(list)
-        for workname, workitem in etcd.get_outstanding_jobs():
-            # A workname looks like: /sf/queue/sf-3/jobname
-            _, _, phase, node, _ = workname.split('/')
-            if node == 'networknode':
-                continue
-
-            for task in workitem:
-                if isinstance(task, FetchBlobTask):
-                    if node in absent_nodes:
-                        LOG.with_fields({
-                            'blob': task.blob_uuid,
-                            'node': node,
-                            'phase': phase
-                        }).warning('Node is absent, ignoring fetch')
-                    else:
-                        LOG.with_fields({
-                            'blob': task.blob_uuid,
-                            'node': node,
-                            'phase': phase
-                        }).info('Node is fetching blob')
-                        current_fetches[task.blob_uuid].append(node)
+        # We count fetches currently requested (or under way) as having completed
+        # in order to stop over-replication for large blobs.
+        current_fetches = etcd.get_current_blob_transfers(
+            absent_nodes=absent_nodes)
 
+        in_use_blobs = []
         with etcd.ThreadLocalReadOnlyCache():
             for b in blob.Blobs([active_states_filter]):
+                if b.instances:
+                    in_use_blobs.append(b)
+
                 # If there is current work for a blob, we ignore it until that
                 # work completes
                 if b.uuid in current_fetches:
                     LOG.with_fields({
-                        'blob': task.blob_uuid
+                        'blob': b.blob_uuid
                     }).info('Blob has current fetches, ignoring')
                     continue
 
@@ -190,6 +198,10 @@ class Monitor(daemon.Daemon):
                             underreplicated.append((b.uuid, 1))
                             break
 
+        # Record blobs in use
+        for b in in_use_blobs:
+            b.record_usage()
+
         # Prune over replicated blobs
         for blob_uuid in overreplicated:
             b = blob.Blob.from_db(blob_uuid)
@@ -219,6 +231,24 @@ class Monitor(daemon.Daemon):
             b.request_replication(allow_excess=excess)
             current_fetches[blob_uuid].append('unknown')
 
+        # Find transcodes of not recently used blobs and reap them
+        old_transcodes = []
+        with etcd.ThreadLocalReadOnlyCache():
+            for b in blob.Blobs([active_states_filter]):
+                if not b.transcoded:
+                    continue
+
+                if time.time() - b.last_used > config.BLOB_TRANSCODE_MAXIMUM_IDLE_TIME:
+                    old_transcodes.append((b.uuid, b.transcoded))
+
+        for blob_uuid, transcodes in old_transcodes:
+            b = blob.Blob.from_db(blob_uuid)
+            b.remove_transcodes()
+
+            for transcode in transcodes:
+                tb = blob.Blob.from_db(transcodes[transcode])
+                tb.ref_count_dec()
+
         # Node management
         for n in Nodes([node_inactive_states_filter]):
             age = time.time() - n.last_seen
