@@ -15,17 +15,15 @@ class Monitor(daemon.Daemon):
     def run(self):
         LOG.info('Starting')
 
-        libvirt = util_libvirt.get_libvirt()
-        conn = libvirt.open('qemu:///system')
-        present_cpus, _, _ = conn.getCPUMap()
-
+        present_cpus = util_libvirt.get_cpu_count()
         os.makedirs('/var/run/sf', exist_ok=True)
         util_process.execute(None, (config.API_COMMAND_LINE
                                     % {
                                         'port': config.API_PORT,
                                         'timeout': config.API_TIMEOUT,
                                         'name': daemon.process_name('api'),
-                                        'workers': present_cpus * 4
+                                        'workers': present_cpus * 2 + 1,
+                                        'threads': present_cpus * 2 + 1
                                     }),
                              env_variables=os.environ,
                              check_exit_code=[0, 1, -15])
