@@ -14,7 +14,7 @@ from shakenfist import exceptions
 from shakenfist import instance
 from shakenfist.ipmanager import IPManager
 from shakenfist import logutil
-from shakenfist import net
+from shakenfist import network
 from shakenfist import networkinterface
 from shakenfist.networkinterface import NetworkInterface
 from shakenfist.tasks import (
@@ -38,9 +38,13 @@ EXTRA_VLANS_HISTORY = {}
 
 class Monitor(daemon.WorkerPoolDaemon):
     def _remove_stray_interfaces(self):
-        for n in net.Networks([baseobject.active_states_filter]):
+        for n in network.Networks([baseobject.active_states_filter]):
             t = time.time()
-            for ni in networkinterface.interfaces_for_network(n):
+            for ni_uuid in n.networkinterfaces:
+                ni = NetworkInterface.from_db(ni_uuid)
+                if not ni:
+                    continue
+
                 inst = instance.Instance.from_db(ni.instance_uuid)
                 if not inst:
                     ni.delete()
@@ -86,20 +90,20 @@ class Monitor(daemon.WorkerPoolDaemon):
                         host_networks.append(ni.network_uuid)
         else:
             # For network nodes, its all networks
-            for n in net.Networks([baseobject.active_states_filter]):
+            for n in network.Networks([baseobject.active_states_filter]):
                 host_networks.append(n.uuid)
 
         # Ensure we are on every network we have a host for
-        for network in host_networks:
+        for network_uuid in host_networks:
             try:
-                n = net.Network.from_db(network)
+                n = network.Network.from_db(network_uuid)
                 if not n:
                     continue
 
                 # If this network is in state delete_wait, then we should remove
                 # it if it has no interfaces left.
                 if n.state.value == dbo.STATE_DELETE_WAIT:
-                    if not networkinterface.interfaces_for_network(n):
+                    if not n.networkinterfaces:
                         LOG.with_network(n).info(
                             'Removing stray delete_wait network')
                         etcd.enqueue('networknode', DestroyNetworkTask(n.uuid))
@@ -123,7 +127,12 @@ class Monitor(daemon.WorkerPoolDaemon):
                         # If the network node was missing a network, then that implies
                         # that we also need to re-create all of the floating IPs for
                         # that network.
-                        for ni in networkinterface.interfaces_for_network(n):
+                        for ni_uuid in n.networkinterfaces:
+                            ni = networkinterface.NetworkInterface.from_db(
+                                ni_uuid)
+                            if not ni:
+                                continue
+
                             if ni.floating.get('floating_address'):
                                 LOG.with_fields(
                                     {
@@ -170,7 +179,7 @@ class Monitor(daemon.WorkerPoolDaemon):
 
     def _process_network_workitem(self, log_ctx, workitem):
         log_ctx = log_ctx.with_network(workitem.network_uuid())
-        n = net.Network.from_db(workitem.network_uuid())
+        n = network.Network.from_db(workitem.network_uuid())
         if not n:
             log_ctx.warning('Received work item for non-existent network')
             return
@@ -185,36 +194,29 @@ class Monitor(daemon.WorkerPoolDaemon):
         #
         if isinstance(workitem, RemoveDHCPNetworkTask):
             n.remove_dhcp()
-            db.add_event('network', workitem.network_uuid(),
-                         'network node', 'remove dhcp', None, None)
             return
 
         if isinstance(workitem, RemoveNATNetworkTask):
             n.remove_nat()
-            db.add_event('network', workitem.network_uuid(),
-                         'network node', 'remove nat', None, None)
             return
 
         #
         # Tasks that should NOT operate on a DEAD network
         #
-        if n.is_dead() and n.state.value != net.Network.STATE_DELETE_WAIT:
+        if n.is_dead() and n.state.value != network.Network.STATE_DELETE_WAIT:
             log_ctx.with_fields({'state': n.state,
                                  'workitem': workitem}).info(
                 'Received work item for a dead network and not delete_wait')
             return
 
         if isinstance(workitem, DestroyNetworkTask):
-            interfaces = list(networkinterface.interfaces_for_network(n))
-            if interfaces:
+            if n.networkinterfaces:
                 log_ctx.error(
                     'DestroyNetworkTask for network with interfaces: %s',
-                    [i.uuid for i in interfaces])
+                    n.networkinterfaces)
                 return
             try:
                 n.delete_on_network_node()
-                db.add_event('network', workitem.network_uuid(),
-                             'network node', 'destroy', None, None)
             except exceptions.DeadNetwork as e:
                 log_ctx.with_field('exception', e).warning(
                     'DestroyNetworkTask on dead network')
@@ -232,8 +234,6 @@ class Monitor(daemon.WorkerPoolDaemon):
             try:
                 n.create_on_network_node()
                 n.ensure_mesh()
-                db.add_event('network', workitem.network_uuid(),
-                             'network node', 'deploy', None, None)
             except exceptions.DeadNetwork as e:
                 log_ctx.with_field('exception', e).warning(
                     'DeployNetworkTask on dead network')
@@ -242,15 +242,13 @@ class Monitor(daemon.WorkerPoolDaemon):
             try:
                 n.create_on_network_node()
                 n.ensure_mesh()
-                db.add_event('network', workitem.network_uuid(),
-                             'network node', 'update dhcp', None, None)
             except exceptions.DeadNetwork as e:
                 log_ctx.with_field('exception', e).warning(
                     'UpdateDHCPNetworkTask on dead network')
 
     def _process_networkinterface_workitem(self, log_ctx, workitem):
         log_ctx = log_ctx.with_networkinterface(workitem.interface_uuid())
-        n = net.Network.from_db(workitem.network_uuid())
+        n = network.Network.from_db(workitem.network_uuid())
         if not n:
             log_ctx.warning('Received work item for non-existent network')
             return
@@ -262,7 +260,7 @@ class Monitor(daemon.WorkerPoolDaemon):
             return
 
         # Tasks that should not operate on a dead or delete waiting network
-        if n.is_dead() and n.state.value != net.Network.STATE_DELETE_WAIT:
+        if n.is_dead() and n.state.value != network.Network.STATE_DELETE_WAIT:
             log_ctx.with_fields({'state': n.state,
                                  'workitem': workitem}).info(
                 'Received work item for a completely dead network')
@@ -271,7 +269,6 @@ class Monitor(daemon.WorkerPoolDaemon):
         if isinstance(workitem, DefloatNetworkInterfaceTask):
             n.remove_floating_ip(ni.floating.get('floating_address'), ni.ipv4)
 
-            db.add_event('interface', ni.uuid, 'api', 'defloat', None, None)
             with db.get_lock('ipmanager', None, 'floating', ttl=120, op='Instance defloat'):
                 ipm = IPManager.from_db('floating')
                 ipm.release(ni.floating.get('floating_address'))
@@ -324,7 +321,7 @@ class Monitor(daemon.WorkerPoolDaemon):
             # Collect floating gateways and floating IPs, while ensuring that
             # they are correctly reserved on the floating network as well
             floating_gateways = []
-            for n in net.Networks([baseobject.active_states_filter]):
+            for n in network.Networks([baseobject.active_states_filter]):
                 fg = n.floating_gateway
                 if fg:
                     floating_gateways.append(fg)
