@@ -45,6 +45,10 @@ class NVRAMTemplateMissing(InstanceException):
     pass
 
 
+class InvalidLifecycleState(InstanceException):
+    pass
+
+
 # Scheduler
 class SchedulerException(Exception):
     pass
@@ -172,6 +176,10 @@ class NoInterfaceStatistics(NetworkException):
     pass
 
 
+class NetworkMissing(NetworkException):
+    pass
+
+
 # NetworkInterface
 class NetworkInterfaceException(Exception):
     pass
@@ -198,6 +206,11 @@ class ArtifactHasNoBlobs(ArtifactException):
     pass
 
 
+class ArtifactHasNoNamespace(ArtifactException):
+    pass
+
+
+# Blobs
 class BlobMissing(ArtifactException):
     pass
 
@@ -208,3 +221,11 @@ class BlobDeleted(ArtifactException):
 
 class BlobFetchFailed(Exception):
     pass
+
+
+class BlobDependencyMissing(Exception):
+    pass
+
+
+class BlobsMustHaveContent(Exception):
+    pass
