@@ -12,6 +12,7 @@ import random
 import shutil
 import socket
 import time
+from uuid import uuid4
 
 from shakenfist import artifact
 from shakenfist import baseobject
@@ -25,9 +26,10 @@ from shakenfist import db
 from shakenfist import etcd
 from shakenfist import exceptions
 from shakenfist import logutil
-from shakenfist import net
+from shakenfist.metrics import get_minimum_object_version as gmov
+from shakenfist import network
 from shakenfist import networkinterface
-from shakenfist.tasks import DeleteInstanceTask
+from shakenfist.tasks import DeleteInstanceTask, SnapshotTask
 from shakenfist.util import general as util_general
 from shakenfist.util import image as util_image
 from shakenfist.util import libvirt as util_libvirt
@@ -75,7 +77,8 @@ def _safe_int_cast(i):
 
 class Instance(dbo):
     object_type = 'instance'
-    current_version = 6
+    current_version = 7
+    upgrade_supported = True
 
     # docs/development/state_machine.md has a description of these states.
     STATE_INITIAL_ERROR = 'initial-error'
@@ -122,6 +125,15 @@ class Instance(dbo):
     METADATA_KEY_AFFINITY = 'affinity'
 
     def __init__(self, static_values):
+        if static_values['version'] != self.current_version:
+            upgraded, static_values = self.upgrade(static_values)
+
+            if upgraded and gmov('instance') == self.current_version:
+                etcd.put(self.object_type, None,
+                         static_values.get('uuid'), static_values)
+                LOG.with_field(
+                    self.object_type, static_values['uuid']).info('Online upgrade committed')
+
         super(Instance, self).__init__(static_values.get('uuid'),
                                        static_values.get('version'))
 
@@ -140,20 +152,58 @@ class Instance(dbo):
         self.__nvram_template = static_values.get('nvram_template')
         self.__secure_boot = static_values.get('secure_boot', False)
         self.__machine_type = static_values.get('machine_type', 'pc')
+        self.__side_channels = static_values.get('side_channels', [])
 
         if not self.__disk_spec:
             # This should not occur since the API will filter for zero disks.
-            self.log.error('Found disk spec empty')
             raise exceptions.InstanceBadDiskSpecification()
 
+    @classmethod
+    def upgrade(cls, static_values):
+        changed = False
+        starting_version = static_values.get('version')
+
+        if static_values.get('version') == 3:
+            static_values['configdrive'] = 'openstack-disk'
+            static_values['version'] = 4
+            changed = True
+
+        if static_values.get('version') == 4:
+            static_values['nvram_template'] = None
+            static_values['secure_boot'] = False
+            static_values['version'] = 5
+            changed = True
+
+        if static_values.get('version') == 5:
+            static_values['machine_type'] = 'pc'
+            static_values['version'] = 6
+            changed = True
+
+        if static_values.get('version') == 6:
+            static_values['side_channels'] = []
+            static_values['version'] = 7
+            changed = True
+
+        if changed:
+            LOG.with_fields({
+                cls.object_type: static_values['uuid'],
+                'start_version': starting_version,
+                'final_version': static_values.get('version')
+            }).info('Object online upgraded')
+        return changed, static_values
+
     @classmethod
     def new(cls, name=None, cpus=None, memory=None, namespace=None, ssh_key=None,
             disk_spec=None, user_data=None, video=None, requested_placement=None,
             instance_uuid=None, uefi=False, configdrive=None, nvram_template=None,
-            secure_boot=False, machine_type='pc'):
+            secure_boot=False, machine_type='pc', side_channels=None):
         if not configdrive:
             configdrive = 'openstack-disk'
 
+        # NOTE(mikal): we don't support creating older versions of objects here.
+        # I am not sure if that makes sense (what do you do if you need to make
+        # an older version but the caller requested a feature which requires the
+        # newer version?), but will call it out for now as a gap.
         static_values = {
             'cpus': cpus,
             'disk_spec': disk_spec,
@@ -169,6 +219,7 @@ class Instance(dbo):
             'nvram_template': nvram_template,
             'secure_boot': secure_boot,
             'machine_type': machine_type,
+            'side_channels': side_channels,
 
             'version': cls.current_version
         }
@@ -200,6 +251,10 @@ class Instance(dbo):
             'nvram_template': self.nvram_template,
             'secure_boot': self.secure_boot,
             'machine_type': self.machine_type,
+            'side_channels': self.side_channels,
+            'agent_state': self.agent_state.value,
+            'agent_start_time': self.agent_start_time,
+            'agent_system_boot_time': self.agent_system_boot_time,
 
             'version': self.version,
             'error_message': self.error,
@@ -237,11 +292,23 @@ class Instance(dbo):
         for iface_uuid in self.interfaces:
             ni = networkinterface.NetworkInterface.from_db(iface_uuid)
             if not ni:
-                self.log.with_object(ni).error(
-                    'Network interface missing')
+                self.log.with_object(ni).error('Network interface missing')
             else:
                 i['interfaces'].append(ni.external_view())
 
+        # Mix in details of the configured disks. We don't have all the details
+        # in the block devices structure until _initialize_block_devices() is
+        # called. If not yet configured, we just return None.
+        i['disks'] = []
+        for disk in self.block_devices.get('devices', []):
+            i['disks'].append({
+                'device': disk['device'],
+                'bus': disk['bus'],
+                'size': disk.get('size'),
+                'blob_uuid': disk.get('blob_uuid'),
+                'snapshot_ignores': disk.get('snapshot_ignores')
+            })
+
         return i
 
     # Static values
@@ -301,6 +368,10 @@ class Instance(dbo):
     def machine_type(self):
         return self.__machine_type
 
+    @property
+    def side_channels(self):
+        return self.__side_channels
+
     @property
     def instance_path(self):
         return os.path.join(config.STORAGE_PATH, 'instances', self.uuid)
@@ -336,23 +407,81 @@ class Instance(dbo):
     def block_devices(self):
         return self._db_get_attribute('block_devices')
 
+    # NOTE(mikal): this really should use the newer _add_item / _remove_item
+    # stuff, but I can't immediately think of an online upgrade path for this
+    # so skipping for now.
     @property
     def interfaces(self):
         return self._db_get_attribute('interfaces')
 
+    # NOTE(mikal): this really should use the newer _add_item / _remove_item
+    # stuff, but I can't immediately think of an online upgrade path for this
+    # so skipping for now.
     @interfaces.setter
     def interfaces(self, interfaces):
         self._db_set_attribute('interfaces', interfaces)
 
     @property
     def tags(self):
-        # TODO(andy) Move metadata to a new DBO subclass "DBO with metadata"
+        # TODO(andy): Move metadata to a new DBO subclass "DBO with metadata"
         meta = db.get_metadata('instance', self.uuid)
         if not meta:
             # Gracefully handle malformed instances
             return None
         return meta.get(self.METADATA_KEY_TAGS, None)
 
+    @property
+    def agent_state(self):
+        db_data = self._db_get_attribute('agent_state')
+        if not db_data:
+            return baseobject.State(None, 0)
+        return baseobject.State(**db_data)
+
+    @agent_state.setter
+    def agent_state(self, new_value):
+        orig = self.agent_state
+        if orig.value == new_value:
+            return
+
+        new_state = baseobject.State(new_value, time.time())
+        self._db_set_attribute('agent_state', new_state)
+
+    @property
+    def agent_start_time(self):
+        db_data = self._db_get_attribute('agent_attributes')
+        return db_data.get('start_time')
+
+    @agent_start_time.setter
+    def agent_start_time(self, new_value):
+        with self.get_lock_attr('agent_attributes', 'Update agent attributes'):
+            db_data = self._db_get_attribute('agent_attributes')
+            db_data['start_time'] = new_value
+            self._db_set_attribute('agent_attributes', db_data)
+
+    @property
+    def agent_system_boot_time(self):
+        db_data = self._db_get_attribute('agent_attributes')
+        return db_data.get('system_boot_time')
+
+    @agent_system_boot_time.setter
+    def agent_system_boot_time(self, new_value):
+        with self.get_lock_attr('agent_attributes', 'Update agent attributes'):
+            db_data = self._db_get_attribute('agent_attributes')
+            db_data['system_boot_time'] = new_value
+            self._db_set_attribute('agent_attributes', db_data)
+
+    @property
+    def agent_facts(self):
+        db_data = self._db_get_attribute('agent_attributes')
+        return db_data.get('facts')
+
+    @agent_facts.setter
+    def agent_facts(self, new_value):
+        with self.get_lock_attr('agent_attributes', 'Update agent facts'):
+            db_data = self._db_get_attribute('agent_attributes')
+            db_data['facts'] = new_value
+            self._db_set_attribute('agent_attributes', db_data)
+
     # Implementation
     def _initialize_block_devices(self):
         bus = _get_defaulted_disk_bus(self.disk_spec[0])
@@ -440,7 +569,6 @@ class Instance(dbo):
             placement['placement_attempts'] = placement.get(
                 'placement_attempts', 0) + 1
             self._db_set_attribute('placement', placement)
-            self.add_event('placement', None, None, location)
 
     def enforced_deletes_increment(self):
         with self.get_lock_attr('enforced_deletes',
@@ -470,8 +598,6 @@ class Instance(dbo):
             dbstate['power_state'] = state
             dbstate['power_state_updated'] = time.time()
             self._db_set_attribute('power_state', dbstate)
-            self.add_event('power state changed', '%s -> %s' %
-                           (dbstate['power_state_previous'], state))
 
     # NOTE(mikal): this method is now strictly the instance specific steps for
     # creation. It is assumed that the image sits in local cache already, and
@@ -490,27 +616,25 @@ class Instance(dbo):
         # Create the actual instance. Sometimes on Ubuntu 20.04 we need to wait
         # for port binding to work. Revisiting this is tracked by issue 320 on
         # github.
-        with util_general.RecordedOperation('create domain', self):
-            if not self.power_on():
-                attempts = 0
-                while not self.power_on() and attempts < 5:
-                    self.log.warning(
-                        'Instance required an additional attempt to power on')
-                    time.sleep(5)
-                    attempts += 1
+        if not self.power_on():
+            attempts = 0
+            while not self.power_on() and attempts < 5:
+                self.log.warning(
+                    'Instance required an additional attempt to power on')
+                time.sleep(5)
+                attempts += 1
 
         if self.is_powered_on():
-            self.log.info('Instance now powered on')
             self.state = self.STATE_CREATED
         else:
-            self.log.info('Instance failed to power on')
+            self.add_event2('instance failed to power on')
             self.enqueue_delete_due_error('Instance failed to power on')
 
     def delete(self):
-        # Mark files we used in the image cache as recently used so that they
-        # linger a little for possible future users.
         for disk in self.block_devices.get('devices', []):
             if 'blob_uuid' in disk and disk['blob_uuid']:
+                # Mark files we used in the image cache as recently used so that
+                # they linger a little for possible future users.
                 cached_image_path = util_general.file_permutation_exists(
                     os.path.join(config.STORAGE_PATH,
                                  'image_cache', disk['blob_uuid']),
@@ -518,23 +642,28 @@ class Instance(dbo):
                 if cached_image_path:
                     pathlib.Path(cached_image_path).touch(exist_ok=True)
 
-        with util_general.RecordedOperation('delete domain', self):
-            try:
-                self.power_off()
-
-                nvram_path = os.path.join(self.instance_path, 'nvram')
-                if os.path.exists(nvram_path):
-                    os.unlink(nvram_path)
-                if self.nvram_template:
-                    b = blob.Blob.from_db(self.nvram_template)
+                # Decrement the blob reference count
+                b = blob.Blob.from_db(disk['blob_uuid'])
+                if b:
                     b.ref_count_dec()
 
-                inst = self._get_domain()
+        try:
+            self.power_off()
+
+            nvram_path = os.path.join(self.instance_path, 'nvram')
+            if os.path.exists(nvram_path):
+                os.unlink(nvram_path)
+            if self.nvram_template:
+                b = blob.Blob.from_db(self.nvram_template)
+                b.ref_count_dec()
+
+            with util_libvirt.LibvirtConnection() as lc:
+                inst = lc.get_domain_from_sf_uuid(self.uuid)
                 if inst:
                     inst.undefine()
-            except Exception as e:
-                util_general.ignore_exception(
-                    'instance delete domain %s' % self, e)
+        except Exception as e:
+            util_general.ignore_exception(
+                'instance delete domain %s' % self, e)
 
         with util_general.RecordedOperation('delete disks', self):
             try:
@@ -551,12 +680,6 @@ class Instance(dbo):
         else:
             self.state = self.STATE_DELETED
 
-    def hard_delete(self):
-        etcd.delete('instance', None, self.uuid)
-        db.delete_metadata('instance', self.uuid)
-        etcd.delete_all('attribute/instance', self.uuid)
-        etcd.delete_all('event/instance', self.uuid)
-
     def _allocate_console_port(self):
         node = config.NODE_NAME
         consumed = [value['port']
@@ -600,7 +723,6 @@ class Instance(dbo):
                     'vdi_port': self._allocate_console_port()
                 }
                 self.ports = p
-                self.log.with_fields(p).info('Console ports allocated')
 
     def deallocate_instance_ports(self):
         ports = self.ports
@@ -617,10 +739,8 @@ class Instance(dbo):
 
             # Generate a config drive
             if self.configdrive == 'openstack-disk':
-                with util_general.RecordedOperation('make config drive', self):
-                    self._make_config_drive_openstack_disk(
-                        os.path.join(self.instance_path,
-                                     block_devices['devices'][1]['path']))
+                self._make_config_drive_openstack_disk(
+                    os.path.join(self.instance_path, block_devices['devices'][1]['path']))
 
             # Prepare disks. A this point we have a file for each blob in the image
             # cache at a well known location (the blob uuid with .qcow2 appended).
@@ -638,7 +758,8 @@ class Instance(dbo):
                             artifact.BLOB_URL, disk['blob_uuid'])
                     elif disk.get('base') and not util_general.noneish(disk.get('base')):
                         a = artifact.Artifact.from_url(
-                            artifact.Artifact.TYPE_IMAGE, disk['base'])
+                            artifact.Artifact.TYPE_IMAGE, disk['base'],
+                            namespace=self.namespace, create_if_new=True)
                         mri = a.most_recent_index
 
                         if 'blob_uuid' not in mri:
@@ -659,6 +780,9 @@ class Instance(dbo):
                             raise exceptions.ImageMissingFromCache(
                                 'Image %s is missing' % disk['blob_uuid'])
 
+                        b = blob.Blob.from_db(disk['blob_uuid'])
+                        b.ref_count_inc()
+
                         with util_general.RecordedOperation('detect cdrom images', self):
                             try:
                                 cd = pycdlib.PyCdlib()
@@ -699,13 +823,38 @@ class Instance(dbo):
                             self.log.with_fields(util_general.stat_log_fields(disk['path'])).info(
                                 'COW layer %s created' % disk['path'])
 
-                            # Record the backing store for modern libvirts
-                            disk['backing'] = (
-                                '<backingStore type=\'file\'>\n'
-                                '        <format type=\'qcow2\'/>\n'
-                                '        <source file=\'%s\'/>\n'
-                                '      </backingStore>\n'
-                                % (cached_image_path))
+                            # Record the backing store for modern libvirt. This requires
+                            # walking the chain of dependencies. Backing chains only work
+                            # for qcow2 images. The backing image should already have been
+                            # transcoded as part of the image fetch process.
+                            backing_chain = []
+                            backing_uuid = disk['blob_uuid']
+                            while backing_uuid:
+                                backing_path = os.path.join(
+                                    config.STORAGE_PATH, 'image_cache', backing_uuid + '.qcow2')
+                                backing_chain.append(backing_path)
+                                backing_blob = blob.Blob.from_db(backing_uuid)
+                                backing_uuid = backing_blob.depends_on
+
+                            indent = '      '
+                            disk['backing'] = ''
+                            backing_chain.reverse()
+
+                            for backing_path in backing_chain:
+                                disk['backing'] = (
+                                    '%(indent)s<backingStore type=\'file\'>'
+                                    '%(indent)s  <format type=\'qcow2\'/>'
+                                    '%(indent)s  <source file=\'%(path)s\'/>\n'
+                                    '%(indent)s  %(chain)s\n'
+                                    '%(indent)s</backingStore>\n'
+                                    % {
+                                        'chain': disk['backing'],
+                                        'path': backing_path,
+                                        'indent': indent
+                                    })
+                                indent += '  '
+
+                            disk['backing'] = disk['backing'].lstrip()
 
                     elif not os.path.exists(disk['path']):
                         util_image.create_blank(
@@ -798,7 +947,7 @@ class Instance(dbo):
                     }
                 )
 
-                n = net.Network.from_db(iface.network_uuid)
+                n = network.Network.from_db(iface.network_uuid)
                 nd['networks'].append(
                     {
                         'id': '%s-%s' % (iface.network_uuid, iface.order),
@@ -870,7 +1019,7 @@ class Instance(dbo):
         networks = []
         for iface_uuid in self.interfaces:
             ni = networkinterface.NetworkInterface.from_db(iface_uuid)
-            n = net.Network.from_db(ni.network_uuid)
+            n = network.Network.from_db(ni.network_uuid)
             networks.append(
                 {
                     'macaddr': ni.macaddr,
@@ -895,12 +1044,26 @@ class Instance(dbo):
                 if not b:
                     raise exceptions.NVRAMTemplateMissing(
                         'Blob %s does not exist' % self.nvram_template)
-                b.ensure_local([])
+                b.ensure_local([], instance_object=self)
                 b.ref_count_inc()
                 shutil.copyfile(
                     blob.Blob.filepath(b.uuid), os.path.join(self.instance_path, 'nvram'))
                 nvram_template_attribute = ''
 
+        # Convert side channels into extra devices. Side channels are implemented
+        # as virtio-serial domain sockets on the hypervisor, and serial posts on
+        # the guest.
+        extradevices = []
+        side_channels = self.side_channels
+        if side_channels:
+            for channel in side_channels:
+                extradevices.append("<channel type='unix'>")
+                extradevices.append(
+                    "  <source mode='bind' path='%s/sc-%s'/>" % (self.instance_path, channel))
+                extradevices.append(
+                    "  <target type='virtio' name='%s' state='connected'/>" % channel)
+                extradevices.append("</channel>")
+
         # NOTE(mikal): the database stores memory allocations in MB, but the
         # domain XML takes them in KB. That wouldn't be worth a comment here if
         # I hadn't spent _ages_ finding a bug related to it.
@@ -921,7 +1084,8 @@ class Instance(dbo):
             secure_boot=self.secure_boot,
             nvram_template_attribute=nvram_template_attribute,
             extracommands=block_devices.get('extracommands', []),
-            machine_type=self.machine_type
+            machine_type=self.machine_type,
+            extradevices=extradevices
         )
 
         # Libvirt re-writes the domain XML once loaded, so we store the XML
@@ -932,111 +1096,121 @@ class Instance(dbo):
 
         return x
 
-    def _get_domain(self):
-        libvirt = util_libvirt.get_libvirt()
-        conn = libvirt.open('qemu:///system')
-        try:
-            return conn.lookupByName('sf:' + self.uuid)
-
-        except libvirt.libvirtError:
-            return None
-
     def is_powered_on(self):
-        inst = self._get_domain()
-        if not inst:
-            return 'off'
+        with util_libvirt.LibvirtConnection() as lc:
+            inst = lc.get_domain_from_sf_uuid(self.uuid)
+            if not inst:
+                return 'off'
 
-        libvirt = util_libvirt.get_libvirt()
-        return util_libvirt.extract_power_state(libvirt, inst)
+            return lc.extract_power_state(inst)
 
     def power_on(self):
-        libvirt = util_libvirt.get_libvirt()
-        inst = self._get_domain()
-        if not inst:
-            conn = libvirt.open('qemu:///system')
-            inst = conn.defineXML(self._create_domain_xml())
+        with util_libvirt.LibvirtConnection() as lc:
+            inst = lc.get_domain_from_sf_uuid(self.uuid)
             if not inst:
-                self.enqueue_delete_due_error(
-                    'power on failed to create domain')
-                raise exceptions.NoDomainException()
+                inst = lc.define_xml(self._create_domain_xml())
+                if not inst:
+                    self.enqueue_delete_due_error(
+                        'power on failed to create domain')
+                    raise exceptions.NoDomainException()
 
-        try:
-            inst.create()
-        except libvirt.libvirtError as e:
-            if str(e).startswith('Requested operation is not valid: '
-                                 'domain is already running'):
-                pass
-            elif str(e).find('Failed to find an available port: '
-                             'Address already in use') != -1:
-                self.log.warning('Instance ports clash: %s', e)
+            try:
+                inst.create()
+            except lc.libvirt.libvirtError as e:
+                if str(e).startswith('Requested operation is not valid: '
+                                     'domain is already running'):
+                    pass
+                elif str(e).find('Failed to find an available port: '
+                                 'Address already in use') != -1:
+                    self.log.warning('Instance ports clash: %s', e)
 
-                # Free those ports and pick some new ones
-                ports = self.ports
-                self._free_console_port(ports['console_port'])
-                self._free_console_port(ports['vdi_port'])
+                    # Free those ports and pick some new ones
+                    ports = self.ports
+                    self._free_console_port(ports['console_port'])
+                    self._free_console_port(ports['vdi_port'])
 
-                # We need to delete the nvram file before we can undefine
-                # the domain. This will be recreated by libvirt on the next
-                # attempt.
-                nvram_path = os.path.join(self.instance_path, 'nvram')
-                if os.path.exists(nvram_path):
-                    os.unlink(nvram_path)
+                    # We need to delete the nvram file before we can undefine
+                    # the domain. This will be recreated by libvirt on the next
+                    # attempt.
+                    nvram_path = os.path.join(self.instance_path, 'nvram')
+                    if os.path.exists(nvram_path):
+                        os.unlink(nvram_path)
 
-                inst.undefine()
+                    inst.undefine()
 
-                self.ports = None
-                self.allocate_instance_ports()
-                return False
-            else:
-                self.log.warning('Instance start error: %s', e)
-                return False
+                    self.ports = None
+                    self.allocate_instance_ports()
+                    return False
+                else:
+                    self.add_event2('instance start error',
+                                    extra={'message': str(e)})
+                    return False
 
-        inst.setAutostart(1)
-        self.update_power_state(
-            util_libvirt.extract_power_state(libvirt, inst))
-        self.add_event('poweron', 'complete')
-        return True
+            inst.setAutostart(1)
+            self.update_power_state(lc.extract_power_state(inst))
+            self.add_event2('poweron')
+            return True
 
     def power_off(self):
-        libvirt = util_libvirt.get_libvirt()
-        inst = self._get_domain()
-        if not inst:
-            return
+        with util_libvirt.LibvirtConnection() as lc:
+            inst = lc.get_domain_from_sf_uuid(self.uuid)
+            if not inst:
+                return
 
-        try:
-            inst.destroy()
-        except libvirt.libvirtError as e:
-            if not str(e).startswith('Requested operation is not valid: '
-                                     'domain is not running'):
-                self.log.error('Failed to delete domain: %s', e)
+            try:
+                inst.destroy()
+            except lc.libvirt.libvirtError as e:
+                if not str(e).startswith('Requested operation is not valid: '
+                                         'domain is not running'):
+                    self.log.error('Failed to delete domain: %s', e)
 
-        self.update_power_state('off')
-        self.add_event('poweroff', 'complete')
+            self.update_power_state('off')
+            self.add_event2('poweroff')
 
     def reboot(self, hard=False):
-        libvirt = util_libvirt.get_libvirt()
-        inst = self._get_domain()
-        if not hard:
-            inst.reboot(flags=libvirt.VIR_DOMAIN_REBOOT_ACPI_POWER_BTN)
-        else:
-            inst.reset()
-        self.add_event('reboot', 'complete')
+        with util_libvirt.LibvirtConnection() as lc:
+            inst = lc.get_domain_from_sf_uuid(self.uuid)
+            if not inst:
+                # Not returning a libvirt domain here indicates that the instance
+                # is "powered off" (destroyed in libvirt speak). It doesn't make
+                # sense to reboot a powered off machine
+                raise exceptions.InvalidLifecycleState(
+                    'you cannot reboot a powered off instance')
+
+            if not hard:
+                inst.reboot(flags=lc.libvirt.VIR_DOMAIN_REBOOT_ACPI_POWER_BTN)
+                self.add_event2('soft reboot')
+            else:
+                inst.reset()
+                self.add_event2('hard reboot')
 
     def pause(self):
-        libvirt = util_libvirt.get_libvirt()
-        inst = self._get_domain()
-        inst.suspend()
-        self.update_power_state(
-            util_libvirt.extract_power_state(libvirt, inst))
-        self.add_event('pause', 'complete')
+        with util_libvirt.LibvirtConnection() as lc:
+            inst = lc.get_domain_from_sf_uuid(self.uuid)
+            if not inst:
+                # Not returning a libvirt domain here indicates that the instance
+                # is "powered off" (destroyed in libvirt speak). It doesn't make
+                # sense to reboot a powered off machine
+                raise exceptions.InvalidLifecycleState(
+                    'you cannot pause a powered off instance')
+
+            inst.suspend()
+            self.update_power_state(lc.extract_power_state(inst))
+            self.add_event2('pause')
 
     def unpause(self):
-        libvirt = util_libvirt.get_libvirt()
-        inst = self._get_domain()
-        inst.resume()
-        self.update_power_state(
-            util_libvirt.extract_power_state(libvirt, inst))
-        self.add_event('unpause', 'complete')
+        with util_libvirt.LibvirtConnection() as lc:
+            inst = lc.get_domain_from_sf_uuid(self.uuid)
+            if not inst:
+                # Not returning a libvirt domain here indicates that the instance
+                # is "powered off" (destroyed in libvirt speak). It doesn't make
+                # sense to reboot a powered off machine
+                raise exceptions.InvalidLifecycleState(
+                    'you cannot unpause a powered off instance')
+
+            inst.resume()
+            self.update_power_state(lc.extract_power_state(inst))
+            self.add_event2('unpause')
 
     def get_console_data(self, length):
         console_path = os.path.join(self.instance_path, 'console.log')
@@ -1050,10 +1224,6 @@ class Instance(dbo):
                 offset = max(0, file_length - length)
                 f.seek(offset)
             d = f.read()
-
-        self.log.info(
-            'Client requested %d bytes of console log, returning %d bytes',
-            length, len(d))
         return d
 
     def delete_console_data(self):
@@ -1061,8 +1231,7 @@ class Instance(dbo):
         if not os.path.exists(console_path):
             return
         os.truncate(console_path, 0)
-        self.add_event('console log cleared', None)
-        self.log.info('Console log cleared')
+        self.add_event2('console log cleared')
 
     def enqueue_delete_remote(self, node):
         etcd.enqueue(node, {
@@ -1070,10 +1239,11 @@ class Instance(dbo):
         })
 
     def enqueue_delete_due_error(self, error_msg):
-        self.log.with_field('error', error_msg).info('enqueue_instance_error')
-
         # Error needs to be set immediately so that API clients get
         # correct information. The VM and network tear down can be delayed.
+        if self.state.value == self.STATE_DELETED:
+            return
+
         try:
             self.state = '%s-error' % self.state.value
         except Exception:
@@ -1083,6 +1253,77 @@ class Instance(dbo):
         self.error = error_msg
         self.enqueue_delete_remote(config.NODE_NAME)
 
+    def snapshot(self, all=False, device=None, max_versions=None, thin=False):
+        disks = self.block_devices['devices']
+
+        # Include NVRAM as a snapshot option if we are UEFI booted
+        if self.uefi:
+            disks.append({
+                'type': 'nvram',
+                'device': 'nvram',
+                'path': os.path.join(self.instance_path, 'nvram'),
+                'snapshot_ignores': False
+            })
+
+        # Filter if requested
+        if device:
+            new_disks = []
+            for d in disks:
+                if d['device'] == device:
+                    new_disks.append(d)
+            disks = new_disks
+        elif not all:
+            disks = [disks[0]]
+        self.log.with_field('devices', disks).info('Devices for snapshot')
+
+        out = {}
+        for disk in disks:
+            if disk['snapshot_ignores']:
+                continue
+
+            if disk['type'] not in ['qcow2', 'nvram']:
+                continue
+
+            if not os.path.exists(disk['path']):
+                continue
+
+            a = artifact.Artifact.from_url(
+                artifact.Artifact.TYPE_SNAPSHOT,
+                '%s%s/%s' % (artifact.INSTANCE_URL, self.uuid, disk['device']),
+                max_versions, namespace=self.namespace, create_if_new=True)
+
+            blob_uuid = str(uuid4())
+            entry = a.add_index(blob_uuid)
+
+            out[disk['device']] = {
+                'source_url': a.source_url,
+                'artifact_uuid': a.uuid,
+                'artifact_index': entry['index'],
+                'blob_uuid': blob_uuid
+            }
+
+            if disk['type'] == 'nvram':
+                # These are small and don't use qemu-img to capture, so just
+                # do them now.
+                blob.ensure_blob_path()
+                dest_path = blob.Blob.filepath(blob_uuid)
+                shutil.copyfile(disk['path'], dest_path)
+
+                st = os.stat(dest_path)
+                b = blob.Blob.new(blob_uuid, st.st_size,
+                                  time.time(), time.time())
+                b.observe()
+                a.state = artifact.Artifact.STATE_CREATED
+
+            else:
+                etcd.enqueue(config.NODE_NAME, {
+                    'tasks': [SnapshotTask(self.uuid, disk, a.uuid, blob_uuid,
+                                           thin=thin)],
+                })
+
+        self.add_event2('snapshot', extra=out)
+        return out
+
 
 class Instances(dbo_iter):
     def __iter__(self):
