@@ -5,13 +5,13 @@ from shakenfist import etcd
 from shakenfist import exceptions
 from shakenfist import logutil
 from shakenfist import tasks
-from shakenfist.config import SFConfigBase
-from shakenfist.tests import test_shakenfist
+from shakenfist.config import BaseSettings
+from shakenfist.tests import base
 
 LOG, _ = logutil.setup(__name__)
 
 
-class FakeConfig(SFConfigBase):
+class FakeConfig(BaseSettings):
     NODE_NAME: str = 'thisnode'
     SLOW_LOCK_THRESHOLD: int = 2
 
@@ -19,7 +19,7 @@ class FakeConfig(SFConfigBase):
 fake_config = FakeConfig()
 
 
-class ActualLockTestCase(test_shakenfist.ShakenFistTestCase):
+class ActualLockTestCase(base.ShakenFistTestCase):
     def setUp(self):
         super(ActualLockTestCase, self).setUp()
 
@@ -53,7 +53,7 @@ class ActualLockTestCase(test_shakenfist.ShakenFistTestCase):
 
     @mock.patch('shakenfist.etcd.ActualLock.get_holder',
                 return_value=('foo', '43'))
-    @mock.patch('shakenfist.db.add_event')
+    @mock.patch('shakenfist.eventlog.add_event')
     @mock.patch('time.time',
                 side_effect=[100.0, 101.0, 102.0, 103.0, 104.0, 105.0,
                              106.0, 107.0, 108.0, 109.0, 110.0, 111.0,
@@ -76,18 +76,12 @@ class ActualLockTestCase(test_shakenfist.ShakenFistTestCase):
             mock_sleep.assert_has_calls(
                 [mock.call(1), mock.call(1), mock.call(1), mock.call(1)])
 
-            mock_add_event.assert_has_calls(
-                [mock.call('instance', 'auuid', 'lock', 'acquire',
-                           None, 'Waiting for lock more than threshold'),
-                 mock.call('instance', 'auuid', 'lock', 'acquired',
-                           None, 'Waited 12 seconds for lock')])
-            mock_get_holder.assert_called_with()
-
+        mock_get_holder.assert_called_with()
         mock_release.assert_called_with()
 
     @mock.patch('shakenfist.etcd.ActualLock.get_holder',
                 return_value=('foo', '43'))
-    @mock.patch('shakenfist.db.add_event')
+    @mock.patch('shakenfist.eventlog.add_event')
     @mock.patch('time.time',
                 side_effect=[100.0, 101.0, 102.0, 103.0, 104.0, 105.0,
                              106.0, 107.0, 108.0, 109.0, 110.0, 111.0,
@@ -109,17 +103,11 @@ class ActualLockTestCase(test_shakenfist.ShakenFistTestCase):
         mock_acquire.assert_has_calls([mock.call(), mock.call()])
         mock_sleep.assert_has_calls([mock.call(1), mock.call(1)])
 
-        mock_add_event.assert_has_calls(
-            [mock.call('instance', 'auuid', 'lock', 'acquire', None,
-                       'Waiting for lock more than threshold'),
-             mock.call('instance', 'auuid', 'lock', 'failed', None,
-                       'Failed to acquire lock after 6.00 seconds')])
         mock_get_holder.assert_called_with()
-
         mock_release.assert_not_called()
 
 
-class TaskEncodingETCDtestCase(test_shakenfist.ShakenFistTestCase):
+class TaskEncodingETCDtestCase(base.ShakenFistTestCase):
     @mock.patch('etcd3gw.Etcd3Client.put')
     def test_put_PreflightInstanceTask(self, mock_put):
         etcd.put('objecttype', 'subtype', 'name',
@@ -180,11 +168,12 @@ class TaskEncodingETCDtestCase(test_shakenfist.ShakenFistTestCase):
     @mock.patch('etcd3gw.Etcd3Client.put')
     def test_put_FetchImageTask(self, mock_put):
         etcd.put('objecttype', 'subtype', 'name',
-                 tasks.FetchImageTask('http://server/image'))
+                 tasks.FetchImageTask('http://server/image', namespace='foo'))
 
         path = '/sf/objecttype/subtype/name'
         encoded = '''{
     "instance_uuid": null,
+    "namespace": "foo",
     "task": "image_fetch",
     "url": "http://server/image",
     "version": 1
@@ -193,11 +182,13 @@ class TaskEncodingETCDtestCase(test_shakenfist.ShakenFistTestCase):
 
         etcd.put('objecttype', 'subtype', 'name',
                  tasks.FetchImageTask('http://server/image',
+                                      namespace='foo',
                                       instance_uuid='fake_uuid'))
 
         path = '/sf/objecttype/subtype/name'
         encoded = '''{
     "instance_uuid": "fake_uuid",
+    "namespace": "foo",
     "task": "image_fetch",
     "url": "http://server/image",
     "version": 1
@@ -208,7 +199,7 @@ class TaskEncodingETCDtestCase(test_shakenfist.ShakenFistTestCase):
 #
 # Decode tasks from JSON
 #
-class TaskDecodingETCDtestCase(test_shakenfist.ShakenFistTestCase):
+class TaskDecodingETCDtestCase(base.ShakenFistTestCase):
     """Test that decodeTasks does decode subclasses of QueueTasks.
 
     Only need to check that JSON will convert to QueueTask objects. Testing of
@@ -255,8 +246,8 @@ class TaskDecodingETCDtestCase(test_shakenfist.ShakenFistTestCase):
 #
 # Dequeue tasks from ETCD
 #
-class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
-    @mock.patch('etcd3gw.Etcd3Client.get_prefix', return_value=[(
+class TaskDequeueTestCase(base.ShakenFistTestCase):
+    @mock.patch('shakenfist.etcd.WrappedEtcdClient.get_prefix', return_value=[(
         '''{
             "tasks": [
                         {
@@ -268,7 +259,7 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
             }
         ''',
         {
-            'key': '/somejob'
+            'key': '/1631269187.890441'
         },
     )])
     @mock.patch('shakenfist.etcd.get_lock')
@@ -276,14 +267,14 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
     @mock.patch('etcd3gw.Etcd3Client.delete')
     def test_dequeue_preflight(self, m_delete, m_put, m_get_lock, m_get_prefix):
         jobname, workitem = etcd.dequeue('node01')
-        self.assertEqual('somejob', jobname)
+        self.assertEqual('1631269187.890441', jobname)
         expected = [
             tasks.PreflightInstanceTask('diff_uuid'),
         ]
         self.assertCountEqual(expected, workitem['tasks'])
         self.assertSequenceEqual(expected, workitem['tasks'])
 
-    @mock.patch('etcd3gw.Etcd3Client.get_prefix', return_value=[(
+    @mock.patch('shakenfist.etcd.WrappedEtcdClient.get_prefix', return_value=[(
         '''{
             "tasks": [
                         {
@@ -295,7 +286,7 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
             }
         ''',
         {
-            'key': '/somejob'
+            'key': '/1631269187.890441'
         },
     )])
     @mock.patch('shakenfist.etcd.get_lock')
@@ -303,14 +294,14 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
     @mock.patch('etcd3gw.Etcd3Client.delete')
     def test_dequeue_start(self, m_delete, m_put, m_get_lock, m_get_prefix):
         jobname, workitem = etcd.dequeue('node01')
-        self.assertEqual('somejob', jobname)
+        self.assertEqual('1631269187.890441', jobname)
         expected = [
             tasks.StartInstanceTask('fake_uuid'),
         ]
         self.assertCountEqual(expected, workitem['tasks'])
         self.assertSequenceEqual(expected, workitem['tasks'])
 
-    @mock.patch('etcd3gw.Etcd3Client.get_prefix', return_value=[(
+    @mock.patch('shakenfist.etcd.WrappedEtcdClient.get_prefix', return_value=[(
         '''{
             "tasks": [
                         {
@@ -323,7 +314,7 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
             }
         ''',
         {
-            'key': '/somejob'
+            'key': '/1631269187.890441'
         },
     )])
     @mock.patch('shakenfist.etcd.get_lock')
@@ -331,14 +322,14 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
     @mock.patch('etcd3gw.Etcd3Client.delete')
     def test_dequeue_error(self, m_delete, m_put, m_get_lock, m_get_prefix):
         jobname, workitem = etcd.dequeue('node01')
-        self.assertEqual('somejob', jobname)
+        self.assertEqual('1631269187.890441', jobname)
         expected = [
             tasks.DeleteInstanceTask('fake_uuid'),
         ]
         self.assertCountEqual(expected, workitem['tasks'])
         self.assertSequenceEqual(expected, workitem['tasks'])
 
-    @mock.patch('etcd3gw.Etcd3Client.get_prefix', return_value=[(
+    @mock.patch('shakenfist.etcd.WrappedEtcdClient.get_prefix', return_value=[(
         '''{
             "tasks": [
                         {
@@ -361,7 +352,7 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
             }
         ''',
         {
-            'key': '/somejob'
+            'key': '/1631269187.890441'
         },
     )])
     @mock.patch('shakenfist.etcd.get_lock')
@@ -369,7 +360,7 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
     @mock.patch('etcd3gw.Etcd3Client.delete')
     def test_dequeue_multi(self, m_delete, m_put, m_get_lock, m_get_prefix):
         jobname, workitem = etcd.dequeue('node01')
-        self.assertEqual('somejob', jobname)
+        self.assertEqual('1631269187.890441', jobname)
         expected = [
             tasks.PreflightInstanceTask('diff_uuid'),
             tasks.StartInstanceTask('fake_uuid'),
@@ -378,7 +369,7 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
         self.assertCountEqual(expected, workitem['tasks'])
         self.assertSequenceEqual(expected, workitem['tasks'])
 
-    @mock.patch('etcd3gw.Etcd3Client.get_prefix', return_value=[(
+    @mock.patch('shakenfist.etcd.WrappedEtcdClient.get_prefix', return_value=[(
         '''{
             "tasks": [
                         {
@@ -391,7 +382,7 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
             }
         ''',
         {
-            'key': '/somejob'
+            'key': '/1631269187.890441'
         },
     )])
     @mock.patch('shakenfist.etcd.get_lock')
@@ -399,18 +390,19 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
     @mock.patch('etcd3gw.Etcd3Client.delete')
     def test_dequeue_delete(self, m_delete, m_put, m_get_lock, m_get_prefix):
         jobname, workitem = etcd.dequeue('node01')
-        self.assertEqual('somejob', jobname)
+        self.assertEqual('1631269187.890441', jobname)
         expected = [
             tasks.DeleteInstanceTask('fake_uuid'),
         ]
         self.assertCountEqual(expected, workitem['tasks'])
         self.assertSequenceEqual(expected, workitem['tasks'])
 
-    @mock.patch('etcd3gw.Etcd3Client.get_prefix', return_value=[(
+    @mock.patch('shakenfist.etcd.WrappedEtcdClient.get_prefix', return_value=[(
         '''{
             "tasks": [
                         {
                             "instance_uuid": "fake_uuid",
+                            "namespace": "foo",
                             "task": "image_fetch",
                             "url": "http://whoknows",
                             "version": 1
@@ -419,7 +411,7 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
             }
         ''',
         {
-            'key': '/somejob'
+            'key': '/1631269187.890441'
         },
     )])
     @mock.patch('shakenfist.etcd.get_lock')
@@ -427,9 +419,10 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
     @mock.patch('etcd3gw.Etcd3Client.delete')
     def test_dequeue_image_fetch(self, m_delete, m_put, m_get_lock, m_get_prefix):
         jobname, workitem = etcd.dequeue('node01')
-        self.assertEqual('somejob', jobname)
+        self.assertEqual('1631269187.890441', jobname)
         expected = [
-            tasks.FetchImageTask('http://whoknows', 'fake_uuid'),
+            tasks.FetchImageTask('http://whoknows', namespace='foo',
+                                 instance_uuid='fake_uuid'),
         ]
         self.assertCountEqual(expected, workitem['tasks'])
         self.assertSequenceEqual(expected, workitem['tasks'])
@@ -438,10 +431,10 @@ class TaskDequeueTestCase(test_shakenfist.ShakenFistTestCase):
 #
 # General ETCD operations
 #
-class GeneralETCDtestCase(test_shakenfist.ShakenFistTestCase):
+class GeneralETCDtestCase(base.ShakenFistTestCase):
     maxDiff = None
 
-    @mock.patch('etcd3gw.Etcd3Client.get_prefix',
+    @mock.patch('shakenfist.etcd.WrappedEtcdClient.get_prefix',
                 return_value=[
                     ('''{"checksum": "ed44b9745b8d62bcbbc180b5f36c24bb",
                         "file_version": 1,
