@@ -26,7 +26,8 @@ PRIORITY_HIGH = (2, 0)
 
 
 def execute(locks, command, check_exit_code=[0], env_variables=None,
-            namespace=None, iopriority=None):
+            namespace=None, iopriority=None, cwd=None,
+            suppress_command_logging=False):
     if namespace:
         command = 'ip netns exec %s %s' % (namespace, command)
 
@@ -36,12 +37,13 @@ def execute(locks, command, check_exit_code=[0], env_variables=None,
             command = 'ionice -c %d -n %d %s' % (iopriority[0], iopriority[1],
                                                  command)
 
-    LOG.info('Executing %s with locks %s', command, locks)
+    if not suppress_command_logging:
+        LOG.info('Executing %s with locks %s', command, locks)
 
     if not locks:
         return processutils.execute(
             command, check_exit_code=check_exit_code,
-            env_variables=env_variables, shell=True)
+            env_variables=env_variables, shell=True, cwd=cwd)
 
     else:
         p = multiprocessing.Process(
@@ -53,5 +55,5 @@ def execute(locks, command, check_exit_code=[0], env_variables=None,
                 command, check_exit_code=check_exit_code,
                 env_variables=env_variables, shell=True)
         finally:
-            p.terminate()
+            p.kill()
             p.join()
