@@ -6,7 +6,6 @@ import logging
 import os
 import random
 import re
-from socket import error as socket_error
 import string
 import sys
 import testtools
@@ -76,7 +75,7 @@ class BaseTestCase(testtools.TestCase):
         sys.stderr.write(
             '----------------------- start %s console -----------------------\n'
             % instance_uuid)
-        for line in self.system_client.get_console_data(instance_uuid, -1).split('\n'):
+        for line in self.system_client.get_console_data(instance_uuid, -1).split('\n')[-20:]:
             sys.stderr.write('Instance console: %s\n' % line)
         sys.stderr.write(
             '----------------------- end %s console -----------------------\n'
@@ -124,13 +123,36 @@ class BaseTestCase(testtools.TestCase):
         return self._await_instance_event(
             instance_uuid, 'detected poweroff', after=after)
 
-    def _await_login_prompt(self, instance_uuid, after=None):
-        return self._await_instance_event(
-            instance_uuid, 'trigger', 'login prompt', after)
+    def _await_instance_ready(self, instance_uuid):
+        self._await_agent_state(instance_uuid, ready=True)
 
-    def _await_cloud_init_complete(self, instance_uuid, after=None):
-        return self._await_instance_event(
-            instance_uuid, 'trigger', 'cloud-init complete', after)
+    def _await_instance_not_ready(self, instance_uuid):
+        self._await_agent_state(instance_uuid, ready=False)
+
+    def _await_agent_state(self, instance_uuid, ready=True):
+        # Wait up to 5 minutes for the instance to be created and enter
+        # the desired agent running state
+        if ready:
+            desired = 'ready'
+        else:
+            desired = 'not ready'
+
+        start_time = time.time()
+        while time.time() - start_time < 5 * 60 * NETWORK_PATIENCE_FACTOR:
+            i = self.system_client.get_instance(instance_uuid)
+            if i['state'] == 'error':
+                raise StartException(
+                    'Instance %s failed to start (marked as error state)'
+                    % instance_uuid)
+
+            if i['agent_state'] and i['agent_state'].startswith(desired):
+                return
+            time.sleep(5)
+
+        raise TimeoutException(
+            'Instance %s failed to start and enter the agent %s state '
+            'in five minutes. Agent state is %s.'
+            % (instance_uuid, desired, i['agent_state']))
 
     def _await_instance_event(
             self, instance_uuid, operation, message=None, after=None):
@@ -155,9 +177,9 @@ class BaseTestCase(testtools.TestCase):
                 'Instance %s was not created in a reasonable time (%s)'
                 % (instance_uuid, i))
 
-        # Once created, we shouldn't need more than another 2 minutes for boot.
+        # Once created, we shouldn't need more than another 5 minutes for boot.
         start_time = time.time()
-        while time.time() - start_time < 2 * 60:
+        while time.time() - start_time < 5 * 60:
             for event in self.system_client.get_instance_events(instance_uuid):
                 if after and event['timestamp'] <= after:
                     continue
@@ -168,28 +190,15 @@ class BaseTestCase(testtools.TestCase):
 
             time.sleep(5)
 
-            # If this is a login prompt, then try mashing the console keyboard
-            if message == 'login prompt':
-                try:
-                    s = telnetlib.Telnet(i['node'], i['console_port'], 30)
-                    s.write('\n'.encode('ascii'))
-                    s.close()
-                except socket_error:
-                    pass
-
         self._log_console(instance_uuid)
         self._log_instance_events(instance_uuid)
         raise TimeoutException(
-            'After time %s, instance %s had no event "%s:%s" (waited 10 mins)' % (
+            'After time %s, instance %s had no event "%s:%s"' % (
                 after, instance_uuid, operation, message))
 
     def _await_image_download_success(self, image_uuid, after=None):
         return self._await_image_event(image_uuid, 'fetch', 'success', after)
 
-    def _await_image_download_error(self, image_uuid, after=None):
-        return self._await_image_event(
-            image_uuid, 'fetch', 'DNS error', after)
-
     def _await_image_event(
             self, image_uuid, operation, message=None, after=None):
         start_time = time.time()
@@ -221,10 +230,18 @@ class BaseTestCase(testtools.TestCase):
 
         while waiting_for:
             for idx, item in copy.copy(waiting_for):
-                n = callback(item)
-                if n.get('state') in ['created', 'deleted', 'error']:
-                    waiting_for.remove((idx, item))
-                    results[idx] = n
+                try:
+                    n = callback(item)
+                    if n.get('state') in ['created', 'deleted', 'error']:
+                        waiting_for.remove((idx, item))
+                        results[idx] = n
+
+                except apiclient.ResourceNotFoundException:
+                    # Its likely this exception can be removed once PR #1314 (or
+                    # equivalent) is merged. The issue right now is that blobs
+                    # aren't created in the database until they're ready on disk,
+                    # which means they initially 404 here.
+                    pass
 
             if waiting_for:
                 time.sleep(5)
@@ -256,6 +273,10 @@ class BaseTestCase(testtools.TestCase):
         return self._await_objects_ready(
             self.system_client.get_artifact, artifact_uuids)
 
+    def _await_blobs_ready(self, blob_uuids):
+        return self._await_objects_ready(
+            self.system_client.get_blob, blob_uuids)
+
     def _test_ping(self, instance_uuid, network_uuid, ip, expected, attempts=1):
         packet_loss_re = re.compile(r'.* ([0-9\.]+)% packet loss.*')
 
@@ -374,7 +395,7 @@ class BaseNamespacedTestCase(BaseTestCase):
             for net in non_blocking_client.get_networks():
                 try:
                     non_blocking_client.delete_network(net['uuid'])
-                except apiclient.ResourceCannotBeDeletedException:
+                except apiclient.ResourceStateConflictException:
                     pass
 
             time.sleep(5)
@@ -412,9 +433,9 @@ class TestDistroBoots(BaseNamespacedTestCase):
                     'base': base_image,
                     'type': 'disk'
                 }
-            ], None, None)
+            ], None, None, side_channels=['sf-agent'])
 
-        self._await_login_prompt(inst['uuid'])
+        self._await_instance_ready(inst['uuid'])
 
         ip = self.test_client.get_instance_interfaces(inst['uuid'])[0]['ipv4']
         self._test_ping(inst['uuid'], self.net['uuid'], ip, 0)
